@@ -1,6 +1,6 @@
 from findpeaks.findpeaks import findpeaks
 
-from findpeaks.utils.smoothline import interpolate_line1d, interpolate_line2d
+from findpeaks.utils.interpolate import interpolate_line1d, interpolate_line2d
 import findpeaks.utils.stats as stats
 
 # Import the denosing filters
@@ -15,7 +15,7 @@ from findpeaks.filters.mean import mean_filter
 
 __author__ = 'Erdogan Tasksen'
 __email__ = 'erdogant@gmail.com'
-__version__ = '2.0.0'
+__version__ = '2.1.1'
 
 # module level doc-string
 __doc__ = """
@@ -39,21 +39,21 @@ Examples
 >>>
 >>> # 2D array example
 >>> from findpeaks import findpeaks
+>>> fp = findpeaks(method='topology')
 >>> X = fp.import_example('2dpeaks')
 >>> results = fp.fit(X)
 >>> fp.plot()
 >>>
 >>> # Image example
 >>> from findpeaks import findpeaks
->>> fp = findpeaks(denoise='fastnl', h=30, resize=(300,300))
+>>> fp = findpeaks(method='topology', denoise='fastnl', window=30, imsize=(300,300))
 >>> X = fp.import_example('2dpeaks_image')
 >>> results = fp.fit(X)
 >>> fp.plot()
 >>>
 >>> # Plot each seperately
 >>> fp.plot_preprocessing()
->>> fp.plot_mask()
->>> fp.plot_peristence()
+>>> fp.plot_persistence()
 >>> fp.plot_mesh()
 
 References
